const categorie = require('./data/categorie.json');
const preferenze = require("./data/preferenze alimentari.json")
const priorities = require("./data/priorities.json")

const SUB_MATRIX = {
	"a": ["e"],
	"e": ["i"],
	"o": ["i"],
	"i": ["e", "o"],
	"u": ["u"]
}

const capitalized = (s) => {
  if (typeof s !== 'string') return ''
  return s.charAt(0).toUpperCase() + s.slice(1).toLowerCase()
}

const preProcess = (items) => {
	const output = []
	const singulars = []
	items = items.map(v => capitalized(v));
	items.forEach((v) => {
		const singular = v.substring(0, v.length - 1);
		if(output.indexOf(v) === -1 && singulars.indexOf(singular) === -1) {
			output.push(v);
			singulars.push(singular)
		}
	})
	return processPriorities(output);
}
const changeSuffix = (val, suffixes) => {
	const prefix = val.substring(0, val.length - 1)
	return [val, ...suffixes.map((v) => prefix + v)]
}
const normalizeToken = (token) => {
	if(token == "") return ""
	token = token.toLowerCase();
	const lastChar = token.charAt(token.length - 1)
	const PLURAL = SUB_MATRIX[lastChar]
	if(PLURAL != null) return changeSuffix(" " + token, PLURAL)
	return [" " + token + " "];
}
const processPriorities = (items) => {
	items.forEach((key) => {
		const rule = priorities[key];
		if(rule) {
			rule.forEach((priority) => {
				items = items.filter((v) => v != priority);
			})

		}
	})
	return items;
}
const filterCriteria = (raw) => {
	return (v) => {
		if(v instanceof Array) {
			const normalizedTokens = v.map((v) => normalizeToken(v));
			for(var i in normalizedTokens) {
				const normalized = normalizedTokens[i]
				if(normalized.filter((v) => raw.indexOf(v) != -1).length > 0) return true;
			}
			return false;
		} else {
			const normalized = normalizeToken(v);
			return normalized.filter((v) => raw.indexOf(v) != -1).length > 0
		}
	}
}
const transform = (includedTokens) => {
	return includedTokens.map((v) => {
		return (v instanceof Array) ? v[0]:  v;
	})
}
const extract = (name, description, portata, set, insertSons, insertParent = true) => {
	if(name == null && description == null && portata == null || set == null) return [];
	var vals = " " + (name || "") + " " + (description || "") + " " + (portata || "") + " ";
	const raw = vals.replace(/[.,&_*#?!%+“`/\<>…@:;()"'-]/g, " ")
	              	  .replace(/\s+/g, " ")
	              	  .toLowerCase();
	var output = [];
	Object.keys(set).forEach((key) => {
		const item = set[key];
		const includedTokens = matchesForKey(item, raw);
		if(includedTokens && includedTokens.length > 0) {
			if(insertParent) output.push(key);
			if(insertSons) output.push(...transform(includedTokens))
		}
	})
	
	return output;
};
const matchesForKey = (item, raw) => {
	const excludedTokens = item.exclude.filter(filterCriteria(raw));
	const includedTokens = item.include.filter(filterCriteria(raw));
	const viable = includedTokens.length > 0 && excludedTokens.length == 0
	return viable ? includedTokens : null;
}
exports.extractCategories = (name, description, portata) => {
	return preProcess([
		...extract(name, description, portata, categorie, true),
		...extract(name, description, portata, preferenze, true, false)
	])
};
exports.extractPreferences = (name, description, portata) => {
	const preferences = extract(name, description, portata, preferenze, false);
	const categories =  extract(name, description, portata, categorie, false);
	return preProcess([
		...preferences,
		...categories
	]).filter((v) => preferences.indexOf(v) != -1)
}


test = (name, description, portata) => {
	console.log("--------------------------------");
	console.log("NAME: ", name);
	console.log("DESCRIPTION: ", description);
	console.log("PORTATA: ", portata);
	console.log("CATEGORIES: ", exports.extractCategories(name, description, portata).join(", "));
	console.log("PREFERENZE: ", exports.extractPreferences(name, description, portata).join(", "));
	console.log("--------------------------------");
}
//test("Tagliolini san daniele", "Ciliegia Cappasanta", "Whiskey melograna");

//test("Titolo", "Insalata mista con pomodorini, patate lesse, uova sode, tonno, acciughe e olive");

//test("Titolo", "Wiener schnitzel artigianale austriaca servita con patate fritte e confettura ai mirtilli - 400gr circa");

//test("Titolo", "Una barretta di vero Parmigiano Reggiano DOP in una comoda confezione monodose");